import nodetool.nodes.huggingface.audio.generate
import nodetool.nodes.huggingface.image.generate
import nodetool.nodes.huggingface.image.segment
import nodetool.nodes.huggingface.image.classify
import nodetool.nodes.huggingface.text.classify
import nodetool.nodes.huggingface.text.summarize
import nodetool.nodes.huggingface.text.generate
