import nodetool.nodes.replicate.audio.enhance
import nodetool.nodes.replicate.audio.generate
import nodetool.nodes.replicate.audio.transcribe
import nodetool.nodes.replicate.audio.separate
import nodetool.nodes.replicate.image.analyze
import nodetool.nodes.replicate.image.enhance
import nodetool.nodes.replicate.image.face
import nodetool.nodes.replicate.image.generate
import nodetool.nodes.replicate.image.ocr
import nodetool.nodes.replicate.image.process
import nodetool.nodes.replicate.image.upscale
import nodetool.nodes.replicate.text.generate
import nodetool.nodes.replicate.video.analyze
import nodetool.nodes.replicate.video.generate
